@@ -8,4 +8,4 @@ distro_string = linux_distribution(supported_dists=('redhat', 'fedora',
 plat_fedora = distro_string in ['redhat', 'fedora']
 plat_debian = distro_string in ['debian']
 logging.basicConfig(format='%(levelname)s:%(funcName)s:%(message)s',
-                    level=logging.DEBUG)
+                    level=logging.INFO)
