@@ -14,6 +14,7 @@ from binascii import hexlify
 
 from tests.fips import fips_mode
 
+
 class RC4TestCase(unittest.TestCase):
 
     @unittest.skipIf(fips_mode, "Can't be run in FIPS mode")
@@ -21,10 +22,12 @@ class RC4TestCase(unittest.TestCase):
         """
         Test with test vectors from Wikipedia: http://en.wikipedia.org/wiki/Rc4
         """
-        vectors = (('Key', 'Plaintext', 'BBF316E8D940AF0AD3'),
-                   ('Wiki', 'pedia', '1021BF0420'),
-                   ('Secret', 'Attack at dawn',
-                    '45A01F645FC35B383552544B9BF5'))
+        if fips_mode:
+            return
+        vectors = ((b'Key', b'Plaintext', b'BBF316E8D940AF0AD3'),
+                   (b'Wiki', b'pedia', b'1021BF0420'),
+                   (b'Secret', b'Attack at dawn',
+                    b'45A01F645FC35B383552544B9BF5'))
 
         rc4 = RC4.RC4()
         for key, plaintext, ciphertext in vectors:
@@ -36,8 +39,10 @@ class RC4TestCase(unittest.TestCase):
 
     @unittest.skipIf(fips_mode, "Can't be run in FIPS mode")
     def test_bad(self):
-        rc4 = RC4.RC4('foo')
-        self.assertNotEqual(hexlify(rc4.update('bar')).upper(), '45678')
+        if fips_mode:
+            return
+        rc4 = RC4.RC4(b'foo')
+        self.assertNotEqual(hexlify(rc4.update(b'bar')).upper(), b'45678')
 
 
 def suite():
