@@ -21,24 +21,32 @@ Others:
 """
 import logging
 import os
+import os.path
 import signal
 import socket
+import subprocess
 import sys
 import tempfile
 import time
+import warnings
 try:
     import unittest2 as unittest
 except ImportError:
     import unittest
 
-from M2Crypto import Err, Rand, SSL, m2
-from tests import plat_debian, plat_fedora
+from M2Crypto import (Err, Rand, SSL, X509, ftpslib, httpslib, m2, m2urllib,
+                      m2urllib2, m2xmlrpclib, util)
+from tests import plat_fedora
 from tests.fips import fips_mode
 
 log = logging.getLogger('test_SSL')
 
+# FIXME
+# It would be probably better if the port was randomly selected.
+# https://fedorahosted.org/libuser/browser/tests/alloc_port.c
 srv_host = 'localhost'
 
+
 def allocate_srv_port():
     s = socket.socket()
     try:
@@ -49,6 +57,7 @@ def allocate_srv_port():
         s.close()
     return port
 
+
 def verify_cb_new_function(ok, store):
     try:
         assert not ok
@@ -58,7 +67,9 @@ def verify_cb_new_function(ok, store):
                        m2.X509_V_ERR_CERT_UNTRUSTED,
                        m2.X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE]
         assert store.get_error_depth() == 0
-        app_data = m2.x509_store_ctx_get_app_data(store.ctx)
+        # app_data = m2.x509_store_ctx_get_app_data(store.ctx)
+        app_data = m2.x509_store_ctx_get_ex_data(
+            store.ctx, m2.ssl_get_ex_data_x509_store_ctx_idx())
         assert app_data
         x509 = store.get_current_cert()
         assert x509
@@ -71,11 +82,13 @@ def verify_cb_new_function(ok, store):
         return 0
     return 1
 
+
 class VerifyCB:
     def __call__(self, ok, store):
         return verify_cb_new_function(ok, store)
 
-sleepTime = float(os.getenv('M2CRYPTO_TEST_SSL_SLEEP', 1.5))
+sleepTime = float(os.getenv('M2CRYPTO_TEST_SSL_SLEEP', '1.5'))
+
 
 def find_openssl():
     if os.name == 'nt' or sys.platform == 'cygwin':
@@ -93,6 +106,7 @@ def find_openssl():
             pass
     return False
 
+
 class BaseSSLClientTestCase(unittest.TestCase):
 
     openssl_in_path = find_openssl()
@@ -101,27 +115,21 @@ class BaseSSLClientTestCase(unittest.TestCase):
         if not self.openssl_in_path:
             raise Exception('openssl command not in PATH')
 
-        pid = os.fork()
-        if pid == 0:
-            # openssl must be started in the tests directory for it
-            # to find the .pem files
-            os.chdir('tests')
-            try:
-                os.execvp('openssl', args)
-            finally:
-                os.chdir('..')
-
-        else:
-            time.sleep(sleepTime)
-            return pid
+        pid = subprocess.Popen(['openssl'] + args,
+                               cwd='tests',
+                               stdout=subprocess.PIPE,
+                               stderr=subprocess.PIPE)
+        time.sleep(sleepTime)
+        return pid
 
     def stop_server(self, pid):
-        os.kill(pid, signal.SIGTERM)
-        os.waitpid(pid, 0)
+        pid.terminate()
+        out, err = pid.communicate()
+        return out, err
 
     def http_get(self, s):
         s.send('GET / HTTP/1.0\n\n')
-        resp = ''
+        resp = b''
         while 1:
             try:
                 r = s.recv(4096)
@@ -130,7 +138,7 @@ class BaseSSLClientTestCase(unittest.TestCase):
             except SSL.SSLError:  # s_server throws an 'unexpected eof'...
                 break
             resp = resp + r
-        return resp
+        return util.py3str(resp)
 
     def setUp(self):
         self.srv_host = srv_host
@@ -147,24 +155,23 @@ class PassSSLClientTestCase(BaseSSLClientTestCase):
     def test_pass(self):
         pass
 
+
 class HttpslibSSLClientTestCase(BaseSSLClientTestCase):
 
     def test_HTTPSConnection(self):
         pid = self.start_server(self.args)
         try:
-            from M2Crypto import httpslib
             c = httpslib.HTTPSConnection(srv_host, self.srv_port)
             c.request('GET', '/')
             data = c.getresponse().read()
             c.close()
         finally:
             self.stop_server(pid)
-        self.assertIn('s_server -quiet -www', data)
+        self.assertIn('s_server -quiet -www', util.py3str(data))
 
     def test_HTTPSConnection_resume_session(self):
         pid = self.start_server(self.args)
         try:
-            from M2Crypto import httpslib
             ctx = SSL.Context()
             ctx.load_verify_locations(cafile='tests/ca.pem')
             ctx.load_cert('tests/x509.pem')
@@ -203,7 +210,6 @@ class HttpslibSSLClientTestCase(BaseSSLClientTestCase):
     def test_HTTPSConnection_secure_context(self):
         pid = self.start_server(self.args)
         try:
-            from M2Crypto import httpslib
             ctx = SSL.Context()
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,
                            9)
@@ -220,79 +226,37 @@ class HttpslibSSLClientTestCase(BaseSSLClientTestCase):
     def test_HTTPSConnection_secure_context_fail(self):
         pid = self.start_server(self.args)
         try:
-            from M2Crypto import httpslib
             ctx = SSL.Context()
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,
                            9)
             ctx.load_verify_locations('tests/server.pem')
             c = httpslib.HTTPSConnection(srv_host, self.srv_port,
                                          ssl_context=ctx)
-            self.assertRaises(SSL.SSLError, c.request, 'GET', '/')
+            with self.assertRaises(SSL.SSLError):
+                c.request('GET', '/')
             c.close()
         finally:
             self.stop_server(pid)
 
-    def test_HTTPS(self):
-        pid = self.start_server(self.args)
+    def test_HTTPSConnection_SNI_support(self):
+        args = ['s_server', '-servername', srv_host, '-debug', '-www',
+                '-cert', 'server.pem', '-key', 'server_key.pem',
+                '-cert2', 'server.pem', '-key2', 'server_key.pem',
+                '-accept', str(self.srv_port)]
+        pid = self.start_server(args)
         try:
-            from M2Crypto import httpslib
-            c = httpslib.HTTPS(srv_host, self.srv_port)
-            c.putrequest('GET', '/')
-            c.putheader('Accept', 'text/html')
-            c.putheader('Accept', 'text/plain')
-            c.endheaders()
-            err, msg, headers = c.getreply()
-            self.assertEqual(err, 200, err)
-            f = c.getfile()
-            data = f.read()
-            c.close()
-        finally:
-            self.stop_server(pid)
-        self.assertIn('s_server -quiet -www', data)
-
-    def test_HTTPS_secure_context(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import httpslib
             ctx = SSL.Context()
-            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,
-                           9)
-            ctx.load_verify_locations('tests/ca.pem')
-            c = httpslib.HTTPS(srv_host, self.srv_port, ssl_context=ctx)
-            c.putrequest('GET', '/')
-            c.putheader('Accept', 'text/html')
-            c.putheader('Accept', 'text/plain')
-            c.endheaders()
-            err, msg, headers = c.getreply()
-            self.assertEqual(err, 200, err)
-            f = c.getfile()
-            data = f.read()
+            c = httpslib.HTTPSConnection(srv_host, self.srv_port,
+                                         ssl_context=ctx)
+            c.request('GET', '/')
             c.close()
         finally:
-            self.stop_server(pid)
-        self.assertIn('s_server -quiet -www', data)
-
-    def test_HTTPS_secure_context_fail(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import httpslib
-            ctx = SSL.Context()
-            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,
-                           9)
-            ctx.load_verify_locations('tests/server.pem')
-            c = httpslib.HTTPS(srv_host, self.srv_port, ssl_context=ctx)
-            c.putrequest('GET', '/')
-            c.putheader('Accept', 'text/html')
-            c.putheader('Accept', 'text/plain')
-            self.assertRaises(SSL.SSLError, c.endheaders)
-            c.close()
-        finally:
-            self.stop_server(pid)
+            out, err = self.stop_server(pid)
+        self.assertIn('Hostname in TLS extension: "%s"' % srv_host, out)
 
     def test_HTTPSConnection_illegalkeywordarg(self):
-        from M2Crypto import httpslib
-        self.assertRaises(ValueError, httpslib.HTTPSConnection, 'example.org',
-                          badKeyword=True)
+        with self.assertRaises(ValueError):
+            httpslib.HTTPSConnection('example.org', badKeyword=True)
 
 
 class MiscSSLClientTestCase(BaseSSLClientTestCase):
@@ -304,11 +268,13 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
     def test_server_simple(self):
         pid = self.start_server(self.args)
         try:
-            self.assertRaises(ValueError, SSL.Context, 'tlsv5')
+            with self.assertRaises(ValueError):
+                SSL.Context('tlsv5')
             ctx = SSL.Context()
             s = SSL.Connection(ctx)
             s.connect(self.srv_addr)
-            self.assertRaises(ValueError, s.read, 0)
+            with self.assertRaises(ValueError):
+                s.read(0)
             data = self.http_get(s)
             s.close()
         finally:
@@ -338,7 +304,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
                            9)
             ctx.load_verify_locations('tests/server.pem')
             s = SSL.Connection(ctx)
-            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            with self.assertRaises(SSL.SSLError):
+                s.connect(self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
@@ -346,7 +313,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
     def test_server_simple_timeouts(self):
         pid = self.start_server(self.args)
         try:
-            self.assertRaises(ValueError, SSL.Context, 'tlsv5')
+            with self.assertRaises(ValueError):
+                SSL.Context('tlsv5')
             ctx = SSL.Context()
             s = SSL.Connection(ctx)
 
@@ -381,10 +349,9 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
         try:
             ctx = SSL.Context('tlsv1')
             s = SSL.Connection(ctx)
-            try:
+            with self.assertRaisesRegexp(SSL.SSLError,
+                                         r'version|unexpected eof'):
                 s.connect(self.srv_addr)
-            except SSL.SSLError as e:
-                self.assertIn(e[0], ['wrong version number', 'unexpected eof'])
             s.close()
         finally:
             self.stop_server(pid)
@@ -404,7 +371,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
 
     # TLS is required in FIPS mode
     @unittest.skipIf(fips_mode, "Can't be run in FIPS mode")
-    @unittest.skipIf(plat_debian, "Debian distros don't allow weak ciphers")
+    @unittest.skipUnless(hasattr(m2, "sslv2_method"),
+                         "This platform doesn't support SSLv2")
     def test_sslv23_weak_crypto(self):
         self.args = self.args + ['-ssl2']
         pid = self.start_server(self.args)
@@ -416,7 +384,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
                 s.connect(self.srv_addr)
                 self.assertEqual(s.get_version(), 'SSLv2')
             else:
-                self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+                with self.assertRaises(SSL.SSLError):
+                    s.connect(self.srv_addr)
             s.close()
         except Exception as ex:
             print(('Caught exception %s' % ex))
@@ -431,10 +400,9 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             ctx = SSL.Context()
             s = SSL.Connection(ctx)
             s.set_cipher_list('AES128-SHA')
-            try:
+            with self.assertRaisesRegexp(SSL.SSLError,
+                                         'sslv3 alert handshake failure'):
                 s.connect(self.srv_addr)
-            except SSL.SSLError as e:
-                self.assertEqual(e[0], 'sslv3 alert handshake failure')
             s.close()
         finally:
             self.stop_server(pid)
@@ -446,10 +414,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             ctx = SSL.Context()
             s = SSL.Connection(ctx)
             s.set_cipher_list('EXP-RC2-MD5')
-            try:
+            with self.assertRaisesRegexp(SSL.SSLError, 'no ciphers available'):
                 s.connect(self.srv_addr)
-            except SSL.SSLError as e:
-                self.assertEqual(e[0], 'no ciphers available')
             s.close()
         finally:
             self.stop_server(pid)
@@ -470,8 +436,10 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             cipher_stack = s.get_ciphers()
             self.assertEqual(cipher_stack[0].name(), 'AES128-SHA',
                              cipher_stack[0].name())
+
             with self.assertRaises(IndexError):
                 cipher_stack.__getitem__(2)
+
             # For some reason there are 2 entries in the stack
             # self.assertEqual(len(cipher_stack), 1, len(cipher_stack))
             self.assertEqual(s.get_cipher_list(), 'AES128-SHA',
@@ -506,7 +474,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -523,7 +491,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -540,7 +508,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -557,7 +525,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -574,18 +542,17 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,
                            9, self.verify_cb_exception)
             s = SSL.Connection(ctx)
-            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            with self.assertRaises(SSL.SSLError):
+                s.connect(self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
 
     def test_verify_cb_not_callable(self):
         ctx = SSL.Context()
-        self.assertRaises(TypeError,
-                          ctx.set_verify,
-                          SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,
-                          9,
-                          1)
+        with self.assertRaises(TypeError):
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,
+                           9, 1)
 
     def test_verify_cb_wrong_callable(self):
         pid = self.start_server(self.args)
@@ -594,19 +561,20 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert,
                            9, lambda _: '')
             s = SSL.Connection(ctx)
-            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            with self.assertRaises(SSL.SSLError):
+                s.connect(self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
 
     def verify_cb_old(self, ctx_ptr, x509_ptr, err, depth, ok):
         try:
-            from M2Crypto import X509
-            assert not ok
-            assert err == m2.X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT or \
-                err == m2.X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY or \
-                err == m2.X509_V_ERR_CERT_UNTRUSTED or \
-                err == m2.X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE
+            self.assertFalse(ok)
+            self.assertIn(err,
+                          [m2.X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT,
+                           m2.X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY,
+                           m2.X509_V_ERR_CERT_UNTRUSTED,
+                           m2.X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE])
             self.assertTrue(m2.ssl_ctx_get_cert_store(ctx_ptr))
             self.assertTrue(X509.X509(x509_ptr).as_pem())
         except AssertionError:
@@ -625,7 +593,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -661,7 +629,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -679,7 +647,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -694,7 +662,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
                            9)
             ctx.load_verify_locations('tests/server.pem')
             s = SSL.Connection(ctx)
-            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            with self.assertRaises(SSL.SSLError):
+                s.connect(self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
@@ -712,7 +681,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -732,7 +701,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -748,7 +717,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
                            9)
             ctx.load_verify_locations('tests/ca.pem')
             s = SSL.Connection(ctx)
-            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            with self.assertRaises(SSL.SSLError):
+                s.connect(self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
@@ -762,7 +732,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
                            9)
             ctx.load_verify_locations('tests/ca.pem')
             s = SSL.Connection(ctx)
-            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            with self.assertRaises(SSL.SSLError):
+                s.connect(self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
@@ -773,7 +744,8 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             ctx = SSL.Context()
             s = SSL.Connection(ctx)
             s.setblocking(0)
-            self.assertRaises(Exception, s.connect, self.srv_addr)
+            with self.assertRaises(Exception):
+                s.connect(self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
@@ -787,7 +759,7 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             data = self.http_get(s)
             s.close()
         finally:
@@ -802,10 +774,10 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
-            bio = s.makefile('rw')
+                self.fail(e)
+            bio = s.makefile('rwb')
             # s.close()  # XXX bug 6628?
-            bio.write('GET / HTTP/1.0\n\n')
+            bio.write(b'GET / HTTP/1.0\n\n')
             bio.flush()
             data = bio.read()
             bio.close()
@@ -822,16 +794,17 @@ class MiscSSLClientTestCase(BaseSSLClientTestCase):
             try:
                 s.connect(self.srv_addr)
             except SSL.SSLError as e:
-                assert 0, e
+                self.fail(e)
             f = s.makefile()
             data = self.http_get(s)
             s.close()
             del f
             del s
             err_code = Err.peek_error_code()
-            assert not err_code, 'Unexpected error: %s' % err_code
+            self.assertEqual(err_code, 0,
+                             'Unexpected error: %s' % err_code)
             err = Err.get_error()
-            assert not err, 'Unexpected error: %s' % err
+            self.assertIsNone(err, 'Unexpected error: %s' % err)
         finally:
             self.stop_server(pid)
         self.assertIn('s_server -quiet -www', data)
@@ -855,8 +828,9 @@ class UrllibSSLClientTestCase(BaseSSLClientTestCase):
     def test_urllib(self):
         pid = self.start_server(self.args)
         try:
-            from M2Crypto import m2urllib
-            url = m2urllib.FancyURLopener()
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore", DeprecationWarning)
+                url = m2urllib.FancyURLopener()
             url.addheader('Connection', 'close')
             u = url.open('https://%s:%s/' % (srv_host, self.srv_port))
             data = u.read()
@@ -865,10 +839,6 @@ class UrllibSSLClientTestCase(BaseSSLClientTestCase):
             self.stop_server(pid)
         self.assertIn('s_server -quiet -www', data)
 
-    # XXX Don't actually know how to use m2urllib safely!
-    # def test_urllib_secure_context(self):
-    # def test_urllib_secure_context_fail(self):
-
     # XXX Don't actually know how to use m2urllib safely!
     # def test_urllib_safe_context(self):
     # def test_urllib_safe_context_fail(self):
@@ -879,7 +849,6 @@ class Urllib2SSLClientTestCase(BaseSSLClientTestCase):
     def test_urllib2(self):
         pid = self.start_server(self.args)
         try:
-            from M2Crypto import m2urllib2
             opener = m2urllib2.build_opener()
             opener.addheaders = [('Connection', 'close')]
             u = opener.open('https://%s:%s/' % (srv_host, self.srv_port))
@@ -897,7 +866,6 @@ class Urllib2SSLClientTestCase(BaseSSLClientTestCase):
                 SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
             ctx.load_verify_locations('tests/ca.pem')
 
-            from M2Crypto import m2urllib2
             opener = m2urllib2.build_opener(ctx)
             opener.addheaders = [('Connection', 'close')]
             u = opener.open('https://%s:%s/' % (srv_host, self.srv_port))
@@ -915,11 +883,10 @@ class Urllib2SSLClientTestCase(BaseSSLClientTestCase):
                 SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
             ctx.load_verify_locations('tests/server.pem')
 
-            from M2Crypto import m2urllib2
             opener = m2urllib2.build_opener(ctx)
             opener.addheaders = [('Connection', 'close')]
-            self.assertRaises(SSL.SSLError, opener.open,
-                              'https://%s:%s/' % (srv_host, self.srv_port))
+            with self.assertRaises(SSL.SSLError):
+                opener.open('https://%s:%s/' % (srv_host, self.srv_port))
         finally:
             self.stop_server(pid)
 
@@ -928,7 +895,6 @@ class Urllib2SSLClientTestCase(BaseSSLClientTestCase):
         try:
             ctx = SSL.Context()
 
-            from M2Crypto import m2urllib2
             opener = m2urllib2.build_opener(
                 ctx, m2urllib2.HTTPBasicAuthHandler())
             m2urllib2.install_opener(opener)
@@ -944,14 +910,12 @@ class Urllib2SSLClientTestCase(BaseSSLClientTestCase):
     def test_urllib2_opener_handlers(self):
         ctx = SSL.Context()
 
-        from M2Crypto import m2urllib2
         m2urllib2.build_opener(ctx, m2urllib2.HTTPBasicAuthHandler())
 
     def test_urllib2_leak(self):
         pid = self.start_server(self.args)
         try:
             import gc
-            from M2Crypto import m2urllib2
             o = m2urllib2.build_opener()
             r = o.open('https://%s:%s/' % (srv_host, self.srv_port))
             s = [r.fp._sock.fp]
@@ -961,6 +925,8 @@ class Urllib2SSLClientTestCase(BaseSSLClientTestCase):
             self.stop_server(pid)
 
 
+@unittest.skipUnless(util.py27plus,
+                     "Twisted doesn't test well with Python 2.6")
 class TwistedSSLClientTestCase(BaseSSLClientTestCase):
 
     def test_timeout(self):
@@ -970,7 +936,8 @@ class TwistedSSLClientTestCase(BaseSSLClientTestCase):
             s = SSL.Connection(ctx)
             # Just a really small number so we can timeout
             s.settimeout(0.000000000000000000000000000001)
-            self.assertRaises(SSL.SSLTimeoutError, s.connect, self.srv_addr)
+            with self.assertRaises(SSL.SSLTimeoutError):
+                s.connect(self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
@@ -979,17 +946,15 @@ class TwistedSSLClientTestCase(BaseSSLClientTestCase):
         # httpslib uses makefile to read the response
         pid = self.start_server(self.args)
         try:
-            from M2Crypto import httpslib
-            c = httpslib.HTTPS(srv_host, self.srv_port)
+            c = httpslib.HTTPSConnection(srv_host, self.srv_port)
             c.putrequest('GET', '/')
             c.putheader('Accept', 'text/html')
             c.putheader('Accept', 'text/plain')
             c.endheaders()
-            c._conn.sock.settimeout(100)
-            err, msg, headers = c.getreply()
-            self.assertEqual(err, 200, err)
-            f = c.getfile()
-            data = f.read()
+            c.sock.settimeout(100)
+            resp = c.getresponse()
+            self.assertEqual(resp.status, 200, resp.reason)
+            data = resp.read()
             c.close()
         finally:
             self.stop_server(pid)
@@ -1007,25 +972,22 @@ class TwistedSSLClientTestCase(BaseSSLClientTestCase):
         try:
             if pipe_pid == 0:
                 try:
-                    f = open('tests/' + FIFO_NAME, 'w')
-                    try:
+                    with open('tests/' + FIFO_NAME, 'w') as f:
                         time.sleep(sleepTime + 1)
                         f.write('Content\n')
-                    finally:
-                        f.close()
                 finally:
                     os._exit(0)
             self.args[self.args.index('-www')] = '-WWW'
             pid = self.start_server(self.args)
             try:
-                from M2Crypto import httpslib
-                c = httpslib.HTTPS(srv_host, self.srv_port)
+                c = httpslib.HTTPSConnection(srv_host, self.srv_port)
                 c.putrequest('GET', '/' + FIFO_NAME)
                 c.putheader('Accept', 'text/html')
                 c.putheader('Accept', 'text/plain')
                 c.endheaders()
-                c._conn.sock.settimeout(0.0000000001)
-                self.assertRaises(socket.timeout, c.getreply)
+                c.sock.settimeout(0.0000000001)
+                with self.assertRaises(socket.timeout):
+                    c.getresponse()
                 c.close()
             finally:
                 self.stop_server(pid)
@@ -1047,9 +1009,10 @@ class TwistedSSLClientTestCase(BaseSSLClientTestCase):
                 'Skipping twisted wrapper test because twisted not found')
             return
 
+        # TODO Class must implement all abstract methods
         class EchoClient(LineReceiver):
             def connectionMade(self):
-                self.sendLine('GET / HTTP/1.0\n\n')
+                self.sendLine(b'GET / HTTP/1.0\n\n')
 
             def lineReceived(self, line):
                 global twisted_data
@@ -1060,7 +1023,7 @@ class TwistedSSLClientTestCase(BaseSSLClientTestCase):
 
             def clientConnectionFailed(self, connector, reason):
                 reactor.stop()
-                assert 0, reason
+                self.fail(reason)
 
             def clientConnectionLost(self, connector, reason):
                 reactor.stop()
@@ -1073,7 +1036,7 @@ class TwistedSSLClientTestCase(BaseSSLClientTestCase):
 
         try:
             global twisted_data
-            twisted_data = ''
+            twisted_data = b''
 
             context_factory = ContextFactory()
             factory = EchoClientFactory()
@@ -1091,30 +1054,28 @@ twisted_data = ''
 
 class XmlRpcLibTestCase(unittest.TestCase):
     def test_lib(self):
-        from M2Crypto import m2xmlrpclib
         m2xmlrpclib.SSL_Transport()
         # XXX need server to test against
 
 
 class FtpsLibTestCase(unittest.TestCase):
     def test_lib(self):
-        from M2Crypto import ftpslib
         ftpslib.FTP_TLS()
         # XXX need server to test against
 
 
 class SessionTestCase(unittest.TestCase):
     def test_session_load_bad(self):
-        self.assertRaises(SSL.SSLError, SSL.Session.load_session,
-                          'tests/signer.pem')
+        with self.assertRaises(SSL.SSLError):
+            SSL.Session.load_session('tests/signer.pem')
+
 
 class FtpslibTestCase(unittest.TestCase):
     def test_26_compat(self):
-        from M2Crypto import ftpslib
         f = ftpslib.FTP_TLS()
         # 2.6 used to raise AttributeError:
-        self.assertRaises(socket.gaierror, f.connect,
-                          'no-such-host-dfgHJK56789', 990)
+        with self.assertRaises(socket.gaierror):
+            f.connect('no-such-host-dfgHJK56789', 990)
 
 
 def suite():
@@ -1129,8 +1090,9 @@ def suite():
     suite.addTest(unittest.makeSuite(MiscSSLClientTestCase))
     suite.addTest(unittest.makeSuite(FtpslibTestCase))
     try:
-        import M2Crypto.SSL.TwistedProtocolWrapper as wrapper  # noqa
-        suite.addTest(unittest.makeSuite(TwistedSSLClientTestCase))
+        if util.py27plus:
+            import M2Crypto.SSL.TwistedProtocolWrapper as wrapper  # noqa
+            suite.addTest(unittest.makeSuite(TwistedSSLClientTestCase))
     except ImportError:
         pass
     return suite
@@ -1141,16 +1103,15 @@ def zap_servers():
     fn = tempfile.mktemp()
     cmd = 'ps | egrep %s > %s' % (s, fn)
     os.system(cmd)
-    f = open(fn)
-    while 1:
-        ps = f.readline()
-        if not ps:
-            break
-        chunk = ps.split()
-        pid, cmd = chunk[0], chunk[4]
-        if cmd == s:
-            os.kill(int(pid), signal.SIGTERM)
-    f.close()
+    with open(fn) as f:
+        while 1:
+            ps = f.readline()
+            if not ps:
+                break
+            chunk = ps.split()
+            pid, cmd = chunk[0], chunk[4]
+            if cmd == s:
+                os.kill(int(pid), signal.SIGTERM)
     os.unlink(fn)
 
 
