@@ -10,22 +10,19 @@ Author: Heikki Toivonen
 """
 
 import base64
+import logging
 import os
-import sys
 import time
 try:
     import unittest2 as unittest
 except ImportError:
     import unittest
-
-# Python 2 has int() and long().
-# Python 3 and higher only has int().
-# Work around this.
-if sys.version_info > (3,):
-    long = int
+import warnings
 
 from M2Crypto import ASN1, BIO, EVP, RSA, Rand, X509, m2  # noqa
 
+log = logging.getLogger(__name__)
+
 
 class X509TestCase(unittest.TestCase):
 
@@ -74,6 +71,10 @@ class X509TestCase(unittest.TestCase):
         self.assertEqual(ext.get_value(flag=m2.X509V3_EXT_PARSE_UNKNOWN),
                          'DNS:foobar.example.com')
 
+    def test_ext_error(self):
+        with self.assertRaises(X509.X509Error):
+            X509.new_extension('nonsensicalName', 'blabla')
+
     def test_extstack(self):
         # new
         ext1 = X509.new_extension('subjectAltName', 'DNS:foobar.example.com')
@@ -237,75 +238,79 @@ class X509TestCase(unittest.TestCase):
         self.assertEqual(req.get_version(), 0)
 
     def test_mkcert(self):
-        req, pk = self.mkreq(1024)
-        pkey = req.get_pubkey()
-        self.assertTrue(req.verify(pkey))
-        sub = req.get_subject()
-        self.assertEqual(len(sub), 2,
-                         'Subject should be long 2 items not %d' % len(sub))
+        for utc in (True, False):
+            req, pk = self.mkreq(1024)
+            pkey = req.get_pubkey()
+            self.assertTrue(req.verify(pkey))
+            sub = req.get_subject()
+            self.assertEqual(len(sub), 2,
+                             'Subject should be long 2 items not %d' % len(sub))
 
-        cert = X509.X509()
-        cert.set_serial_number(1)
-        cert.set_version(2)
-        cert.set_subject(sub)
-        t = long(time.time()) + time.timezone
-        now = ASN1.ASN1_UTCTIME()
-        now.set_time(t)
-        now_plus_year = ASN1.ASN1_UTCTIME()
-        now_plus_year.set_time(t + 60 * 60 * 24 * 365)
-        cert.set_not_before(now)
-        cert.set_not_after(now_plus_year)
-        self.assertEqual(str(cert.get_not_before()), str(now))
-        self.assertEqual(str(cert.get_not_after()), str(now_plus_year))
+            cert = X509.X509()
+            cert.set_serial_number(1)
+            cert.set_version(2)
+            cert.set_subject(sub)
+            t = int(time.time()) + time.timezone
+            if utc:
+                now = ASN1.ASN1_UTCTIME()
+            else:
+                now = ASN1.ASN1_TIME()
+            now.set_time(t)
+            now_plus_year = ASN1.ASN1_TIME()
+            now_plus_year.set_time(t + 60 * 60 * 24 * 365)
+            cert.set_not_before(now)
+            cert.set_not_after(now_plus_year)
+            self.assertEqual(str(cert.get_not_before()), str(now))
+            self.assertEqual(str(cert.get_not_after()), str(now_plus_year))
 
-        issuer = X509.X509_Name()
-        issuer.CN = 'The Issuer Monkey'
-        issuer.O = 'The Organization Otherwise Known as My CA, Inc.'
-        cert.set_issuer(issuer)
-        cert.set_pubkey(pkey)
-        cert.set_pubkey(cert.get_pubkey())  # Make sure get/set work
+            issuer = X509.X509_Name()
+            issuer.CN = 'The Issuer Monkey'
+            issuer.O = 'The Organization Otherwise Known as My CA, Inc.'
+            cert.set_issuer(issuer)
+            cert.set_pubkey(pkey)
+            cert.set_pubkey(cert.get_pubkey())  # Make sure get/set work
 
-        ext = X509.new_extension('subjectAltName', 'DNS:foobar.example.com')
-        ext.set_critical(0)
-        self.assertEqual(ext.get_critical(), 0)
-        cert.add_ext(ext)
+            ext = X509.new_extension('subjectAltName', 'DNS:foobar.example.com')
+            ext.set_critical(0)
+            self.assertEqual(ext.get_critical(), 0)
+            cert.add_ext(ext)
 
-        cert.sign(pk, 'sha1')
-        with self.assertRaises(ValueError):
-            cert.sign(pk, 'nosuchalgo')
+            cert.sign(pk, 'sha1')
+            with self.assertRaises(ValueError):
+                cert.sign(pk, 'nosuchalgo')
 
-        self.assertTrue(cert.get_ext('subjectAltName').get_name(),
-                        'subjectAltName')
-        self.assertTrue(cert.get_ext_at(0).get_name(),
-                        'subjectAltName')
-        self.assertTrue(cert.get_ext_at(0).get_value(),
-                        'DNS:foobar.example.com')
-        self.assertEqual(cert.get_ext_count(), 1,
-                         'Certificate should have now 1 extension not %d' %
-                         cert.get_ext_count())
-        with self.assertRaises(IndexError):
-            cert.get_ext_at(1)
-        self.assertTrue(cert.verify())
-        self.assertTrue(cert.verify(pkey))
-        self.assertTrue(cert.verify(cert.get_pubkey()))
-        self.assertEqual(cert.get_version(), 2)
-        self.assertEqual(cert.get_serial_number(), 1)
-        self.assertEqual(cert.get_issuer().CN, 'The Issuer Monkey')
+            self.assertTrue(cert.get_ext('subjectAltName').get_name(),
+                            'subjectAltName')
+            self.assertTrue(cert.get_ext_at(0).get_name(),
+                            'subjectAltName')
+            self.assertTrue(cert.get_ext_at(0).get_value(),
+                            'DNS:foobar.example.com')
+            self.assertEqual(cert.get_ext_count(), 1,
+                             'Certificate should have now 1 extension not %d' %
+                             cert.get_ext_count())
+            with self.assertRaises(IndexError):
+                cert.get_ext_at(1)
+            self.assertTrue(cert.verify())
+            self.assertTrue(cert.verify(pkey))
+            self.assertTrue(cert.verify(cert.get_pubkey()))
+            self.assertEqual(cert.get_version(), 2)
+            self.assertEqual(cert.get_serial_number(), 1)
+            self.assertEqual(cert.get_issuer().CN, 'The Issuer Monkey')
 
-        if m2.OPENSSL_VERSION_NUMBER >= long(0x90800f):
-            self.assertFalse(cert.check_ca())
-            self.assertFalse(cert.check_purpose(m2.X509_PURPOSE_SSL_SERVER, 1))
-            self.assertFalse(cert.check_purpose(m2.X509_PURPOSE_NS_SSL_SERVER,
-                                                1))
-            self.assertTrue(cert.check_purpose(m2.X509_PURPOSE_SSL_SERVER, 0))
-            self.assertTrue(cert.check_purpose(m2.X509_PURPOSE_NS_SSL_SERVER,
-                                               0))
-            self.assertTrue(cert.check_purpose(m2.X509_PURPOSE_ANY, 0))
-        else:
-            with self.assertRaises(AttributeError):
-                cert.check_ca()
+            if m2.OPENSSL_VERSION_NUMBER >= 0x90800f:
+                self.assertFalse(cert.check_ca())
+                self.assertFalse(cert.check_purpose(m2.X509_PURPOSE_SSL_SERVER, 1))
+                self.assertFalse(cert.check_purpose(m2.X509_PURPOSE_NS_SSL_SERVER,
+                                                    1))
+                self.assertTrue(cert.check_purpose(m2.X509_PURPOSE_SSL_SERVER, 0))
+                self.assertTrue(cert.check_purpose(m2.X509_PURPOSE_NS_SSL_SERVER,
+                                                   0))
+                self.assertTrue(cert.check_purpose(m2.X509_PURPOSE_ANY, 0))
+            else:
+                with self.assertRaises(AttributeError):
+                    cert.check_ca()
 
-    def mkcacert(self):
+    def mkcacert(self, utc):
         req, pk = self.mkreq(1024, ca=1)
         pkey = req.get_pubkey()
         sub = req.get_subject()
@@ -313,10 +318,13 @@ class X509TestCase(unittest.TestCase):
         cert.set_serial_number(1)
         cert.set_version(2)
         cert.set_subject(sub)
-        t = long(time.time()) + time.timezone
-        now = ASN1.ASN1_UTCTIME()
+        t = int(time.time()) + time.timezone
+        if utc:
+            now = ASN1.ASN1_UTCTIME()
+        else:
+            now = ASN1.ASN1_TIME()
         now.set_time(t)
-        now_plus_year = ASN1.ASN1_UTCTIME()
+        now_plus_year = ASN1.ASN1_TIME()
         now_plus_year.set_time(t + 60 * 60 * 24 * 365)
         cert.set_not_before(now)
         cert.set_not_after(now_plus_year)
@@ -348,56 +356,66 @@ class X509TestCase(unittest.TestCase):
         return cert, pk, pkey
 
     def test_mkcacert(self):
-        cacert, _, pkey = self.mkcacert()
-        self.assertTrue(cacert.verify(pkey))
+        for utc in (True, False):
+            cacert, _, pkey = self.mkcacert(utc)
+            self.assertTrue(cacert.verify(pkey))
 
     def test_mkproxycert(self):
-        cacert, pk1, _ = self.mkcacert()
-        end_entity_cert_req, pk2 = self.mkreq(1024)
-        end_entity_cert = self.make_eecert(cacert)
-        end_entity_cert.set_subject(end_entity_cert_req.get_subject())
-        end_entity_cert.set_pubkey(end_entity_cert_req.get_pubkey())
-        end_entity_cert.sign(pk1, 'sha1')
-        proxycert = self.make_proxycert(end_entity_cert)
-        proxycert.sign(pk2, 'sha1')
-        self.assertTrue(proxycert.verify(pk2))
-        self.assertEqual(proxycert.get_ext_at(0).get_name(),
-                         'proxyCertInfo')
-        self.assertEqual(proxycert.get_ext_at(0).get_value(),
-                         'Path Length Constraint: infinite\n' +
-                         'Policy Language: Inherit all\n')
-        self.assertEqual(proxycert.get_ext_count(), 1,
-                         proxycert.get_ext_count())
-        self.assertEqual(proxycert.get_subject().as_text(),
-                         'C=UK, CN=OpenSSL Group, CN=Proxy')
-        self.assertEqual(
-            proxycert.get_subject().as_text(indent=2,
-                                            flags=m2.XN_FLAG_RFC2253),
-            '  CN=Proxy,CN=OpenSSL Group,C=UK')
+        for utc in (True, False):
+            cacert, pk1, _ = self.mkcacert(utc)
+            end_entity_cert_req, pk2 = self.mkreq(1024)
+            end_entity_cert = self.make_eecert(cacert, utc)
+            end_entity_cert.set_subject(end_entity_cert_req.get_subject())
+            end_entity_cert.set_pubkey(end_entity_cert_req.get_pubkey())
+            end_entity_cert.sign(pk1, 'sha1')
+            proxycert = self.make_proxycert(end_entity_cert, utc)
+            proxycert.sign(pk2, 'sha1')
+            self.assertTrue(proxycert.verify(pk2))
+            self.assertEqual(proxycert.get_ext_at(0).get_name(),
+                             'proxyCertInfo')
+            self.assertEqual(proxycert.get_ext_at(0).get_value(),
+                             'Path Length Constraint: infinite\n' +
+                             'Policy Language: Inherit all\n')
+            self.assertEqual(proxycert.get_ext_count(), 1,
+                             proxycert.get_ext_count())
+            self.assertEqual(proxycert.get_subject().as_text(),
+                             'C=UK, CN=OpenSSL Group, CN=Proxy')
+            self.assertEqual(
+                proxycert.get_subject().as_text(indent=2,
+                                                flags=m2.XN_FLAG_RFC2253),
+                '  CN=Proxy,CN=OpenSSL Group,C=UK')
 
-    def make_eecert(self, cacert):
+    @staticmethod
+    def make_eecert(cacert, utc):
         eecert = X509.X509()
         eecert.set_serial_number(2)
         eecert.set_version(2)
-        t = long(time.time()) + time.timezone
-        now = ASN1.ASN1_UTCTIME()
+        t = int(time.time()) + time.timezone
+        if utc:
+            now = ASN1.ASN1_UTCTIME()
+        else:
+            now = ASN1.ASN1_TIME()
         now.set_time(t)
-        now_plus_year = ASN1.ASN1_UTCTIME()
+        now_plus_year = ASN1.ASN1_TIME()
         now_plus_year.set_time(t + 60 * 60 * 24 * 365)
         eecert.set_not_before(now)
         eecert.set_not_after(now_plus_year)
         eecert.set_issuer(cacert.get_subject())
         return eecert
 
-    def make_proxycert(self, eecert):
+    def make_proxycert(self, eecert, utc):
         proxycert = X509.X509()
         pk2 = EVP.PKey()
         proxykey = RSA.gen_key(1024, 65537, self.callback)
         pk2.assign_rsa(proxykey)
         proxycert.set_pubkey(pk2)
         proxycert.set_version(2)
-        not_before = ASN1.ASN1_UTCTIME()
-        not_after = ASN1.ASN1_UTCTIME()
+        if utc:
+            not_before = ASN1.ASN1_UTCTIME()
+            not_after = ASN1.ASN1_UTCTIME()
+        else:
+            not_before = ASN1.ASN1_TIME()
+            not_after = ASN1.ASN1_TIME()
         not_before.set_time(int(time.time()))
         offset = 12 * 3600
         not_after.set_time(int(time.time()) + offset)
@@ -431,17 +449,18 @@ class X509TestCase(unittest.TestCase):
         self.assertEqual(fp, self.expected_hash)
 
     def test_load_der_string(self):
-        f = open('tests/x509.der', 'rb')
-        x509 = X509.load_cert_der_string(''.join(f.readlines()))
+        with open('tests/x509.der', 'rb') as f:
+            x509 = X509.load_cert_der_string(f.read())
+
         fp = x509.get_fingerprint('sha1')
         self.assertEqual(fp, self.expected_hash)
 
     def test_save_der_string(self):
         x509 = X509.load_cert('tests/x509.pem')
         s = x509.as_der()
-        f = open('tests/x509.der', 'rb')
-        s2 = f.read()
-        f.close()
+        with open('tests/x509.der', 'rb') as f:
+            s2 = f.read()
+
         self.assertEqual(s, s2)
 
     def test_load(self):
@@ -453,10 +472,10 @@ class X509TestCase(unittest.TestCase):
         return
 
     def test_load_bio(self):
-        bio = BIO.openfile('tests/x509.pem')
-        bio2 = BIO.openfile('tests/x509.der')
-        x509 = X509.load_cert_bio(bio)
-        x5092 = X509.load_cert_bio(bio2, format=X509.FORMAT_DER)
+        with BIO.openfile('tests/x509.pem') as bio:
+            with BIO.openfile('tests/x509.der') as bio2:
+                x509 = X509.load_cert_bio(bio)
+                x5092 = X509.load_cert_bio(bio2, format=X509.FORMAT_DER)
 
         with self.assertRaises(ValueError):
             X509.load_cert_bio(bio2, format=45678)
@@ -466,12 +485,12 @@ class X509TestCase(unittest.TestCase):
         self.assertEqual(x509.as_der(), x5092.as_der())
 
     def test_load_string(self):
-        f = open('tests/x509.pem')
-        s = f.read()
-        f.close()
-        f2 = open('tests/x509.der', 'rb')
-        s2 = f2.read()
-        f2.close()
+        with open('tests/x509.pem') as f:
+            s = f.read()
+
+        with open('tests/x509.der', 'rb') as f2:
+            s2 = f2.read()
+
         x509 = X509.load_cert_string(s)
         x5092 = X509.load_cert_string(s2, X509.FORMAT_DER)
         self.assertEqual(x509.as_text(), x5092.as_text())
@@ -498,26 +517,26 @@ class X509TestCase(unittest.TestCase):
 
     def test_save(self):
         x509 = X509.load_cert('tests/x509.pem')
-        f = open('tests/x509.pem', 'r')
-        l_tmp = f.readlines()
-        # -----BEGIN CERTIFICATE----- : -----END CERTIFICATE-----
-        beg_idx = l_tmp.index('-----BEGIN CERTIFICATE-----\n')
-        end_idx = l_tmp.index('-----END CERTIFICATE-----\n')
-        x509_pem = ''.join(l_tmp[beg_idx:end_idx + 1])
-        f.close()
-        f = open('tests/x509.der', 'rb')
-        x509_der = f.read()
-        f.close()
+        with open('tests/x509.pem', 'r') as f:
+            l_tmp = f.readlines()
+            # -----BEGIN CERTIFICATE----- : -----END CERTIFICATE-----
+            beg_idx = l_tmp.index('-----BEGIN CERTIFICATE-----\n')
+            end_idx = l_tmp.index('-----END CERTIFICATE-----\n')
+            x509_pem = ''.join(l_tmp[beg_idx:end_idx + 1])
+
+        with open('tests/x509.der', 'rb') as f:
+            x509_der = f.read()
+
         x509.save('tests/tmpcert.pem')
-        f = open('tests/tmpcert.pem')
-        s = f.read()
-        f.close()
+        with open('tests/tmpcert.pem') as f:
+            s = f.read()
+
         self.assertEqual(s, x509_pem)
         os.remove('tests/tmpcert.pem')
         x509.save('tests/tmpcert.der', format=X509.FORMAT_DER)
-        f = open('tests/tmpcert.der', 'rb')
-        s = f.read()
-        f.close()
+        with open('tests/tmpcert.der', 'rb') as f:
+            s = f.read()
+
         self.assertEqual(s, x509_der)
         os.remove('tests/tmpcert.der')
 
@@ -547,17 +566,28 @@ class X509TestCase(unittest.TestCase):
         self.assertEqual(cert.get_serial_number(),
                          127614157056681299805556476275995414779)
 
-    def test_date_after_2050_not_working(self):
+    def test_date_after_2050_working(self):
         cert = X509.load_cert('tests/bad_date_cert.crt')
-        with self.assertRaises(X509.X509Error):
-            cert.get_not_after()
+        self.assertEqual(str(cert.get_not_after()), 'Feb  9 14:57:46 2116 GMT')
+
+    def test_easy_rsa_generated(self):
+        """ Test loading a cert generated by easy RSA.
+
+        https://github.com/fedora-infra/fedmsg/pull/389
+        """
+        # Does this raise an exception?
+        cert = X509.load_cert('tests/easy_rsa.pem')
 
 
 class X509StackTestCase(unittest.TestCase):
     def test_make_stack_from_der(self):
-        f = open("tests/der_encoded_seq.b64")
-        b64 = f.read(1304)
-        seq = base64.decodestring(b64)
+        with open("tests/der_encoded_seq.b64", 'rb') as f:
+            b64 = f.read()
+
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore', DeprecationWarning)
+            seq = base64.decodestring(b64)
+
         stack = X509.new_stack_from_der(seq)
         cert = stack.pop()
         self.assertIsNone(stack.pop())
@@ -571,9 +601,13 @@ class X509StackTestCase(unittest.TestCase):
             "/DC=org/DC=doegrids/OU=Services/CN=host/bosshog.lbl.gov")
 
     def test_make_stack_check_num(self):
-        f = open("tests/der_encoded_seq.b64")
-        b64 = f.read(1304)
-        seq = base64.decodestring(b64)
+        with open("tests/der_encoded_seq.b64", 'rb') as f:
+            b64 = f.read()
+
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore', DeprecationWarning)
+            seq = base64.decodestring(b64)
+
         stack = X509.new_stack_from_der(seq)
         num = len(stack)
         self.assertEqual(num, 1)
@@ -637,9 +671,8 @@ class X509ExtTestCase(unittest.TestCase):
             name = "nsComment"
             value = "Hello"
 
-        lhash = m2.x509v3_lhash()
-        ctx = m2.x509v3_set_conf_lhash(lhash)
-        x509_ext_ptr = m2.x509v3_ext_conf(lhash, ctx, name, value)
+        ctx = m2.x509v3_set_nconf()
+        x509_ext_ptr = m2.x509v3_ext_conf(None, ctx, name, value)
         X509.X509_Extension(x509_ext_ptr, 1)
 
 
